-- Created by: Danny Bloemendaal, danny@bloemeland.nl
-- Version 0.9.3

-- make sure we can find our modules
local SCRIPTFOLDER = 'scripts'
local scriptPath = debug.getinfo(1).source:match("@?(.*/)")
package.path    = package.path .. ';' .. scriptPath .. '?.lua'
package.path    = package.path .. ';' .. scriptPath .. SCRIPTFOLDER .. '/?.lua'


local MAIN_METHOD = 'execute'

-- load the mddule and call the MAIN_METHOD
-- also has some error handling
function callEventHandler(eventHandler, value, name, index)
	if (eventHandler[MAIN_METHOD] ~= nil) then
		local ok2, res = pcall(eventHandler[MAIN_METHOD], value, name, index)
		if (ok2) then
			return res
		else
			print('An error occured when calling event handler ' .. eventHandler)
			print(res) -- error info
		end
	else
		print('No' .. MAIN_METHOD .. 'function found in event handler ' .. eventHandler)
	end
	return {}
end

-- return the index of a given device name
-- strips any additions like _Temperature first
function getIndex(device, otherdevices_idx)
	if (otherdevices_idx == nil) then
		-- old system
		return 0
	end

	-- first get the name, it could be 'My temperature_humidity', we only need the part before the _
	local name = device
	local pos, len = string.find(device, '_')
	if (pos ~= nil and pos > 1) then -- cannot start with _ (we use that for our _always script)
		name = string.sub(device, 1, pos-1)
	end
	-- now find the index in the table otherdevices_idx
	local idx = otherdevices_idx[name]

	return idx
end

function length(t)
	local count = 0
	if (type(t) == 'table') then
		for i,j in pairs(t) do
			count = count + 1
		end
	end
	return count
end

function handleEvents(events, deviceValue, commandArray, deviceName, deviceIndex)
	local commands

	if (type(events) ~= 'table') then
		return commandArray
	end

	for eventIdx, eventHandler in pairs(events) do
		print('=====================================================')
		print('>>> Handler: ' .. eventHandler.name )
		if (deviceName) then
			print('>>> Device: "' .. deviceName .. '" Index: ' .. deviceIndex)
		end
		print('.....................................................')

		commands = callEventHandler(eventHandler, deviceValue, deviceName, deviceIndex)

		print('.....................................................')
		print('<<< Done ')
		print('-----------------------------------------------------')

		-- commandIndex = commandIndex + 1
		for k,v in pairs(commands) do
			if (type(k) == 'number' and type(v) == 'table') then
				table.insert(commandArray, v)
			else
				table.insert(commandArray, {[k]=v})
			end

		end
	end

	return commandArray, commandIndex
end

function getEventBindings(mode)
	local bindings = {}
	local ok, modules, moduleName, i, event, j, device
	ok, modules = pcall( scandir, scriptPath .. '/' .. SCRIPTFOLDER)
	if (not ok) then
		print(modules)
		return nil
	end

	for i, moduleName  in pairs(modules) do
		local module, skip
		ok, module = pcall(require, moduleName)
		if (ok) then
			if (type(module) == 'table') then
				skip = false
				if (module.active ~= nil) then
					local active = (type(module.active) == 'function' and module.active() or module.active)
					if (not active) then
						skip = true
					end
				end
				if (not skip) then
					if ( module.on ~= nil and module[MAIN_METHOD]~= nil ) then
						module.name = moduleName
						for j, event in pairs(module.on) do
							if (mode == 'timer') then
								if (event == 'timer') then
									table.insert(bindings, module)
								end
							else
								if (event ~= 'timer') then
									-- let's not try to resolve indexes to names here for performance reasons
									if ( bindings[event] == nil) then
										bindings[event] = {}
									end
									table.insert(bindings[event], module)
								end
							end
						end
					else
						print('Script ' .. moduleName .. '.lua has no "on" and/or "' .. MAIN_METHOD .. '" section. Skipping')
					end
				end
			else
				print('Script ' .. moduleName .. '.lua is not a valid module. Skipping')
			end
		else
			print(module)
		end
	end

	return bindings
end

function getTimerHandlers()
	return getEventBindings('timer')
end

function scandir(directory)
	local pos, len
	local i, t, popen = 0, {}, io.popen
	local sep = string.sub(package.config,1,1)
	local cmd

	if (sep=='/') then
		cmd = 'ls -a "'..directory..'"'
	else
		-- assume windows for now
		cmd = 'dir "'..directory..'" /b /ad'
	end

	local pfile = popen(cmd)
	for filename in pfile:lines() do

		pos,len = string.find(filename, '.lua')
		if (pos and pos > 0 ) then
			i = i + 1
			t[i] = string.sub(filename, 1, pos-1)
			-- print('module ' .. t[i])
		end

	end
	pfile:close()
	return t
end

function dumpCommandArray(commandArray)
	local printed = false
	for k,v in pairs(commandArray) do
		if (type(v)=='table') then
			for kk,vv in pairs(v) do
				print('[' .. k .. '] = ' .. kk .. ': ' .. vv)
			end
		else
			print(k .. ': ' .. v)
		end
		printed = true
	end
	if(printed) then print('=====================================================') end
end

return {
	callEventHandler = callEventHandler,
	getIndex = getIndex,
	handleEvents = handleEvents,
	getEventBindings = getEventBindings,
	getTimerHandlers = getTimerHandlers,
	dumpCommandArray = dumpCommandArray,
	length = length
}
